@@ -16,6 +16,7 @@ try:
     import boto3
     import botocore.client
     import botocore.exceptions
+    import urllib3.exceptions
 except ImportError:
     MISSING_DEPS = True
 
@@ -130,7 +131,7 @@ def _consolidate_params(uri, transport_params):
     if session is not None and (uri['access_id'] or uri['access_secret']):
         logger.warning(
             'ignoring credentials parsed from URL because they conflict with '
-            'transport_params.session. Set transport_params.session to None '
+            'transport_params["session"]. Set transport_params["session"] to None '
             'to suppress this warning.'
         )
         uri.update(access_id=None, access_secret=None)
@@ -157,7 +158,7 @@ def _override_endpoint_url(transport_params, url):
     if resource_kwargs.get('endpoint_url'):
         logger.warning(
             'ignoring endpoint_url parsed from URL because it conflicts '
-            'with transport_params.resource_kwargs.endpoint_url. '
+            'with transport_params["resource_kwargs"]["endpoint_url"]'
         )
     else:
         resource_kwargs.update(endpoint_url=url)
@@ -260,6 +261,7 @@ def open(
                 key_id,
                 min_part_size=min_part_size,
                 session=session,
+                resource=resource,
                 upload_kwargs=multipart_upload_kwargs,
                 resource_kwargs=resource_kwargs,
             )
@@ -268,6 +270,7 @@ def open(
                 bucket_id,
                 key_id,
                 session=session,
+                resource=resource,
                 upload_kwargs=singlepart_upload_kwargs,
                 resource_kwargs=resource_kwargs,
             )
@@ -307,7 +310,12 @@ class _SeekableRawReader(object):
     This class is internal to the S3 submodule.
     """
 
-    def __init__(self, s3_object, version_id=None, object_kwargs=None):
+    def __init__(
+        self,
+        s3_object,
+        version_id=None,
+        object_kwargs=None,
+    ):
         self._object = s3_object
         self._content_length = None
         self._version_id = version_id
@@ -379,7 +387,6 @@ class _SeekableRawReader(object):
         if start is None and stop is None:
             start = self._position
         range_string = smart_open.utils.make_range_string(start, stop)
-        logger.debug('range_string: %r', range_string)
 
         try:
             # Optimistically try to fetch the requested content range.
@@ -397,18 +404,22 @@ class _SeekableRawReader(object):
             self._position = self._content_length = int(error_response['ActualObjectSize'])
             self._body = io.BytesIO()
         else:
+            #
+            # Keep track of how many times boto3's built-in retry mechanism
+            # activated.
+            #
+            # https://boto3.amazonaws.com/v1/documentation/api/latest/guide/retries.html#checking-retry-attempts-in-an-aws-service-response
+            #
+            logger.debug(
+                '%s: RetryAttempts: %d',
+                self,
+                response['ResponseMetadata']['RetryAttempts'],
+            )
             units, start, stop, length = smart_open.utils.parse_content_range(response['ContentRange'])
             self._content_length = length
             self._position = start
             self._body = response['Body']
 
-    def _read_from_body(self, size=-1):
-        if size == -1:
-            binary = self._body.read()
-        else:
-            binary = self._body.read(size)
-        return binary
-
     def read(self, size=-1):
         """Read from the continuous connection with the remote peer."""
         if self._body is None:
@@ -417,14 +428,50 @@ class _SeekableRawReader(object):
         if self._position >= self._content_length:
             return b''
 
-        try:
-            binary = self._read_from_body(size)
-        except botocore.exceptions.IncompleteReadError:
-            # The underlying connection of the self._body was closed by the remote peer.
-            self._open_body()
-            binary = self._read_from_body(size)
-        self._position += len(binary)
-        return binary
+        #
+        # Boto3 has built-in error handling and retry mechanisms:
+        #
+        # https://boto3.amazonaws.com/v1/documentation/api/latest/guide/error-handling.html
+        # https://boto3.amazonaws.com/v1/documentation/api/latest/guide/retries.html
+        #
+        # Unfortunately, it isn't always enough. There is still a non-zero
+        # possibility that an exception will slip past these mechanisms and
+        # terminate the read prematurely.  Luckily, at this stage, it's very
+        # simple to recover from the problem: wait a little bit, reopen the
+        # HTTP connection and try again.  Usually, a single retry attempt is
+        # enough to recover, but we try multiple times "just in case".
+        #
+        for attempt, seconds in enumerate([1, 2, 4, 8, 16], 1):
+            try:
+                if size == -1:
+                    binary = self._body.read()
+                else:
+                    binary = self._body.read(size)
+            except (
+                ConnectionResetError,
+                botocore.exceptions.BotoCoreError,
+                urllib3.exceptions.HTTPError,
+            ) as err:
+                logger.warning(
+                    '%s: caught %r while reading %d bytes, sleeping %ds before retry',
+                    self,
+                    err,
+                    size,
+                    seconds,
+                )
+                time.sleep(seconds)
+                self._open_body()
+            else:
+                self._position += len(binary)
+                return binary
+
+        raise IOError('%s: failed to read %d bytes after %d attempts' % (self, size, attempt))
+
+    def __str__(self):
+        return 'smart_open.s3._SeekableReader(%r, %r)' % (
+            self._object.bucket_name,
+            self._object.key,
+        )
 
 
 def _initialize_boto3(rw, session, resource, resource_kwargs):
@@ -515,7 +562,6 @@ class Reader(io.BufferedIOBase):
 
     def close(self):
         """Flush and close this stream."""
-        logger.debug("close: called")
         self._object = None
 
     def readable(self):
@@ -594,8 +640,6 @@ class Reader(io.BufferedIOBase):
         :param int whence: Where the offset is from.
 
         Returns the position after seeking."""
-        logger.debug('seeking to offset: %r whence: %r', offset, whence)
-
         # Convert relative offset to absolute, since self._raw_reader
         # doesn't know our current position.
         if whence == constants.WHENCE_CURRENT:
@@ -603,7 +647,6 @@ class Reader(io.BufferedIOBase):
             offset += self._current_pos
 
         self._current_pos = self._raw_reader.seek(offset, whence)
-        logger.debug('new_position: %r', self._current_pos)
 
         self._buffer.empty()
         self._eof = self._current_pos == self._raw_reader._content_length
@@ -647,11 +690,9 @@ class Reader(io.BufferedIOBase):
     #
     def _read_from_buffer(self, size=-1):
         """Remove at most size bytes from our buffer and return them."""
-        # logger.debug('reading %r bytes from %r byte-long buffer', size, len(self._buffer))
         size = size if size >= 0 else len(self._buffer)
         part = self._buffer.read(size)
         self._current_pos += len(part)
-        # logger.debug('part: %r', part)
         return part
 
     def _fill_buffer(self, size=-1):
@@ -659,7 +700,7 @@ class Reader(io.BufferedIOBase):
         while len(self._buffer) < size and not self._eof:
             bytes_read = self._buffer.fill(self._raw_reader)
             if bytes_read == 0:
-                logger.debug('reached EOF while filling buffer')
+                logger.debug('%s: reached EOF while filling buffer', self)
                 self._eof = True
 
     def __str__(self):
@@ -743,14 +784,13 @@ multipart upload may fail")
     # Override some methods from io.IOBase.
     #
     def close(self):
-        logger.debug("closing")
         if self._buf.tell():
             self._upload_next_part()
 
         if self._total_bytes and self._mp:
             partial = functools.partial(self._mp.complete, MultipartUpload={'Parts': self._parts})
             _retry_if_failed(partial)
-            logger.debug("completed multipart upload")
+            logger.debug('%s: completed multipart upload', self)
         elif self._mp:
             #
             # AWS complains with "The XML you provided was not well-formed or
@@ -759,12 +799,11 @@ multipart upload may fail")
             #
             # We work around this by creating an empty file explicitly.
             #
-            logger.info("empty input, ignoring multipart upload")
             assert self._mp, "no multipart upload in progress"
             self._mp.abort()
             self._object.put(Body=b'')
+            logger.debug('%s: wrote 0 bytes to imitate multipart upload', self)
         self._mp = None
-        logger.debug("successfully closed")
 
     @property
     def closed(self):
@@ -823,8 +862,13 @@ multipart upload may fail")
     #
     def _upload_next_part(self):
         part_num = self._total_parts + 1
-        logger.info("uploading part #%i, %i bytes (total %.3fGB)",
-                    part_num, self._buf.tell(), self._total_bytes / 1024.0 ** 3)
+        logger.info(
+            "%s: uploading part_num: %i, %i bytes (total %.3fGB)",
+            self,
+            part_num,
+            self._buf.tell(),
+            self._total_bytes / 1024.0 ** 3,
+        )
         self._buf.seek(0)
         part = self._mp.Part(part_num)
 
@@ -837,7 +881,7 @@ multipart upload may fail")
         upload = _retry_if_failed(functools.partial(part.upload, Body=self._buf))
 
         self._parts.append({'ETag': upload['ETag'], 'PartNumber': part_num})
-        logger.debug("upload of part #%i finished" % part_num)
+        logger.debug("%s: upload of part_num #%i finished", self, part_num)
 
         self._total_parts += 1
         self._buf = io.BytesIO()
@@ -927,7 +971,7 @@ class SinglepartWriter(io.BufferedIOBase):
             raise ValueError(
                 'the bucket %r does not exist, or is forbidden for access' % self._object.bucket_name) from e
 
-        logger.debug("direct upload finished")
+        logger.debug("%s: direct upload finished", self)
         self._buf = None
 
     @property
